AndThen

Struct AndThen 

pub struct AndThen<A, B>(pub A, pub B);
Run a bytes combinator A and then re-interpret the entire bytes consumed/produced by A with another combinator B.

Consistency

A value of type B::Val is consistent w.r.t AndThen<A, B> iff there exists a value of type A::Val that is consistent w.r.t A and whose byte length equals the byte length of the B::Val value w.r.t B. Prefer ExactLen over AndThen to avoid the existential reasoning in the consistency condition.

Tuple Fields

0: A1: B