pub struct AndThen<A, B>(pub A, pub B);Expand description
Run a bytes combinator A and then
re-interpret the entire bytes consumed/produced by A with another combinator B.
Consistency
A value of type B::Val is consistent w.r.t AndThen<A, B> iff there exists a value of type
A::Val that is consistent w.r.t A and whose byte length equals the byte length of the B::Val value w.r.t B.
Prefer ExactLen over AndThen to avoid the existential reasoning in the consistency condition.
Tuple Fields
0: A1: BTrait Implementations
Sourceimpl<A, Then> Consistency for AndThen<A, Then>where
A: BytesCombinator + Consistency<Val = Seq<u8>>,
Then: Consistency + SpecByteLen<T = Then::Val>,
impl<A, Then> Consistency for AndThen<A, Then>where
A: BytesCombinator + Consistency<Val = Seq<u8>>,
Then: Consistency + SpecByteLen<T = Then::Val>,
Sourceopen spec fn consistent(&self, v: Self::Val) -> bool
open spec fn consistent(&self, v: Self::Val) -> bool
{
&&& self.1.consistent(v)
&&& exists |chunk: Seq<u8>| {
self.0.consistent(chunk) && self.0.byte_len(chunk) == self.1.byte_len(v)
}
}Sourcetype Val = <Then as Consistency>::Val
type Val = <Then as Consistency>::Val
The type of values whose consistency is being checked.
Sourceimpl<A, Then> EquivSerializers for AndThen<A, Then>
impl<A, Then> EquivSerializers for AndThen<A, Then>
Sourceimpl<A, Then> EquivSerializersGeneral for AndThen<A, Then>
impl<A, Then> EquivSerializersGeneral for AndThen<A, Then>
Sourceopen spec fn equiv_general_inv(&self) -> bool
open spec fn equiv_general_inv(&self) -> bool
{
&&& self.0.equiv_general_inv()
&&& self.1.equiv_inv()
}Sourceproof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
proof fn lemma_serialize_equiv(&self, v: Self::SVal, obuf: Seq<u8>)
Sourceimpl<A, Then> GoodSerializer for AndThen<A, Then>
impl<A, Then> GoodSerializer for AndThen<A, Then>
Sourceopen spec fn serialize_inv(&self) -> bool
open spec fn serialize_inv(&self) -> bool
{
&&& self.0.serialize_inv()
&&& self.1.serialize_inv()
}Sourceproof fn lemma_serialize_len(&self, v: Self::SVal)
proof fn lemma_serialize_len(&self, v: Self::SVal)
Sourceimpl<A, Then> MinMaxByteLen for AndThen<A, Then>
impl<A, Then> MinMaxByteLen for AndThen<A, Then>
Sourceimpl<A, Then> NoLookAhead for AndThen<A, Then>
impl<A, Then> NoLookAhead for AndThen<A, Then>
Sourceopen spec fn no_lookahead_inv(&self) -> bool
open spec fn no_lookahead_inv(&self) -> bool
{ self.0.no_lookahead_inv() }Sourceproof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
proof fn lemma_no_lookahead(&self, i1: Seq<u8>, i2: Seq<u8>)
Sourceimpl<A, Then> NonMalleable for AndThen<A, Then>
impl<A, Then> NonMalleable for AndThen<A, Then>
Sourceopen spec fn nonmal_inv(&self) -> bool
open spec fn nonmal_inv(&self) -> bool
{
&&& self.0.nonmal_inv()
&&& self.0.sound_inv()
&&& self.1.nonmal_inv()
&&& self.1.safe_inv()
}Sourceproof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
proof fn lemma_parse_non_malleable(&self, buf1: Seq<u8>, buf2: Seq<u8>)
Sourceimpl<A, Then> NonTailFmt for AndThen<A, Then>
impl<A, Then> NonTailFmt for AndThen<A, Then>
Sourceopen spec fn serialize_dps_inv(&self) -> bool
open spec fn serialize_dps_inv(&self) -> bool
{
&&& self.0.serialize_dps_inv()
&&& self.1.serialize_inv()
&&& self.1.equiv_inv()
}Sourceproof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_prepend(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceproof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
proof fn lemma_serialize_dps_len(&self, v: Self::SValue, obuf: Seq<u8>)
Sourceimpl<A, Then> Productive for AndThen<A, Then>
impl<A, Then> Productive for AndThen<A, Then>
Sourceopen spec fn productive_inv(&self) -> bool
open spec fn productive_inv(&self) -> bool
{ self.0.productive_inv() }Sourceproof fn lemma_productive(&self, s: Seq<u8>)
proof fn lemma_productive(&self, s: Seq<u8>)
Sourceimpl<Then> SPRoundTripDps for AndThen<Tail, Then>
impl<Then> SPRoundTripDps for AndThen<Tail, Then>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{
&&& self.1.serialize_inv()
&&& self.1.equiv_inv()
&&& self.1.sp_roundtrip_inv()
}Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<Len, Then> SPRoundTripDps for AndThen<Varied<Len>, Then>
impl<Len, Then> SPRoundTripDps for AndThen<Varied<Len>, Then>
Sourceopen spec fn unambiguous(&self) -> bool
open spec fn unambiguous(&self) -> bool
{
&&& self.1.serialize_inv()
&&& self.1.equiv_inv()
&&& self.1.sp_roundtrip_inv()
}Sourceproof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
proof fn theorem_serialize_dps_parse_roundtrip(&self, v: Self::T, obuf: Seq<u8>)
Sourceimpl<A, Then> SafeParser for AndThen<A, Then>
impl<A, Then> SafeParser for AndThen<A, Then>
Sourceimpl<Output: OutputBuf, Then, T> Serializer<Output, T> for AndThen<Tail, Then>where
T: DeepView + ?Sized,
Then: Serializer<Output, T>,
impl<Output: OutputBuf, Then, T> Serializer<Output, T> for AndThen<Tail, Then>where
T: DeepView + ?Sized,
Then: Serializer<Output, T>,
Sourceexec fn serialize_into(&self, v: &T, obuf: &mut Output)
exec fn serialize_into(&self, v: &T, obuf: &mut Output)
Sourceimpl<A, Then> SoundParser for AndThen<A, Then>
impl<A, Then> SoundParser for AndThen<A, Then>
Sourceimpl<A, Then: SpecByteLen> SpecByteLen for AndThen<A, Then>
impl<A, Then: SpecByteLen> SpecByteLen for AndThen<A, Then>
Sourceimpl<A, Then> SpecParser for AndThen<A, Then>
impl<A, Then> SpecParser for AndThen<A, Then>
Sourceopen spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
open spec fn spec_parse(&self, ibuf: Seq<u8>) -> Option<(int, Self::PVal)>
{
match self.0.spec_parse(ibuf) {
None => None,
Some((len_a, chunk)) => {
match self.1.spec_parse(chunk) {
Some((len_b, v)) if len_a == len_b => Some((len_a, v)),
_ => None,
}
}
}
}Sourcetype PVal = <Then as SpecParser>::PVal
type PVal = <Then as SpecParser>::PVal
The type of parsed values.
Sourceimpl<A, Then> SpecSerializer for AndThen<A, Then>
impl<A, Then> SpecSerializer for AndThen<A, Then>
Sourceopen spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
open spec fn spec_serialize(&self, v: Self::SVal) -> Seq<u8>
{
let inner_bytes = self.1.spec_serialize(v);
self.0.spec_serialize(inner_bytes)
}Sourcetype SVal = <Then as SpecSerializer>::SVal
type SVal = <Then as SpecSerializer>::SVal
The type of values to be serialized.
Sourceimpl<A, Then> SpecSerializerDps for AndThen<A, Then>
impl<A, Then> SpecSerializerDps for AndThen<A, Then>
Sourceimpl<A, Then> ValueByteLen for AndThen<A, Then>
impl<A, Then> ValueByteLen for AndThen<A, Then>
Sourceopen spec fn value_byte_len(v: Self::T) -> nat
open spec fn value_byte_len(v: Self::T) -> nat
{ Then::value_byte_len(v) }Sourceproof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
proof fn lemma_value_len_matches_byte_len(&self, v: Self::T)
impl<A: Copy, B: Copy> Copy for AndThen<A, B>